@@ -12,13 +12,13 @@
 #include <QCoreApplication>
 #include <QDateTime>
 #include <QDirIterator>
+#include <QList>
 #include <QStandardPaths>
-#include <QVector>
 
 #include <KLazyLocalizedString>
-#include <KLocalizedString>
 #include <KPluginFactory>
 #include <KPluginMetaData>
+#include <unordered_set>
 
 #include "config-package.h"
 
@@ -29,40 +29,6 @@
 
 namespace KPackage
 {
-static PackageLoader *s_packageTrader = nullptr;
-
-QSet<QString> PackageLoaderPrivate::s_customCategories;
-
-QSet<QString> PackageLoaderPrivate::knownCategories()
-{
-    // this is to trick the translation tools into making the correct
-    // strings for translation
-    QSet<QString> categories = s_customCategories;
-    // clang-format off
-    categories << kli18n("Accessibility").toString().toLower()
-               << kli18n("Application Launchers").toString().toLower()
-               << kli18n("Astronomy").toString().toLower()
-               << kli18n("Date and Time").toString().toLower()
-               << kli18n("Development Tools").toString().toLower()
-               << kli18n("Education").toString().toLower()
-               << kli18n("Environment and Weather").toString().toLower()
-               << kli18n("Examples").toString().toLower()
-               << kli18n("File System").toString().toLower()
-               << kli18n("Fun and Games").toString().toLower()
-               << kli18n("Graphics").toString().toLower()
-               << kli18n("Language").toString().toLower()
-               << kli18n("Mapping").toString().toLower()
-               << kli18n("Miscellaneous").toString().toLower()
-               << kli18n("Multimedia").toString().toLower()
-               << kli18n("Online Services").toString().toLower()
-               << kli18n("Productivity").toString().toLower()
-               << kli18n("System Information").toString().toLower()
-               << kli18n("Utilities").toString().toLower()
-               << kli18n("Windows and Tasks").toString().toLower();
-    // clang-format on
-    return categories;
-}
-
 PackageLoader::PackageLoader()
     : d(new PackageLoaderPrivate)
 {
@@ -76,49 +42,19 @@ PackageLoader::~PackageLoader()
     delete d;
 }
 
-#if KPACKAGE_BUILD_DEPRECATED_SINCE(5, 86)
-void PackageLoader::setPackageLoader(PackageLoader *loader)
-{
-    if (!s_packageTrader) {
-        s_packageTrader = loader;
-    }
-}
-#endif
-
 PackageLoader *PackageLoader::self()
 {
-    if (!s_packageTrader) {
-        // we have been called before any PackageLoader was set, so just use the default
-        // implementation. this prevents plugins from nefariously injecting their own
-        // plugin loader if the app doesn't
-        s_packageTrader = new PackageLoader;
-        s_packageTrader->d->isDefaultLoader = true;
-    }
-
+    static PackageLoader *s_packageTrader = new PackageLoader;
     return s_packageTrader;
 }
 
 Package PackageLoader::loadPackage(const QString &packageFormat, const QString &packagePath)
 {
-#if KPACKAGE_BUILD_DEPRECATED_SINCE(5, 86)
-    if (!d->isDefaultLoader) {
-        Package p = internalLoadPackage(packageFormat);
-        if (p.hasValidStructure()) {
-            if (!packagePath.isEmpty()) {
-                p.setPath(packagePath);
-            }
-            return p;
-        }
-    }
-#endif
-
     if (packageFormat.isEmpty()) {
         return Package();
     }
 
-    PackageStructure *structure = loadPackageStructure(packageFormat);
-
-    if (structure) {
+    if (PackageStructure *structure = loadPackageStructure(packageFormat)) {
         Package p(structure);
         if (!packagePath.isEmpty()) {
             p.setPath(packagePath);
@@ -126,10 +62,6 @@ Package PackageLoader::loadPackage(const QString &packageFormat, const QString &
         return p;
     }
 
-#ifndef NDEBUG
-    // qCDebug(KPACKAGE_LOG) << "Couldn't load Package for" << packageFormat << "! reason given: " << error;
-#endif
-
     return Package();
 }
 
@@ -144,7 +76,7 @@ QList<KPluginMetaData> PackageLoader::listPackages(const QString &packageFormat,
         d->pluginCache.clear();
     }
 
-    const QString cacheKey = QStringLiteral("%1.%2").arg(packageFormat, packageRoot);
+    const QString cacheKey = packageFormat + QLatin1Char('.') + packageRoot;
     if (useRuntimeCache) {
         auto it = d->pluginCache.constFind(cacheKey);
         if (it != d->pluginCache.constEnd()) {
@@ -164,17 +96,15 @@ QList<KPluginMetaData> PackageLoader::listPackages(const QString &packageFormat,
     if (actualRoot.isEmpty()) {
         PackageStructure *structure = d->structures.value(packageFormat).data();
         if (!structure) {
-            if (packageFormat == QStringLiteral("KPackage/Generic")) {
+            if (packageFormat == QLatin1String("KPackage/Generic")) {
                 structure = new GenericPackage();
-            } else if (packageFormat == QStringLiteral("KPackage/GenericQML")) {
+            } else if (packageFormat == QLatin1String("KPackage/GenericQML")) {
                 structure = new GenericQMLPackage();
+            } else {
+                structure = loadPackageStructure(packageFormat);
             }
         }
 
-        if (!structure) {
-            structure = loadPackageStructure(packageFormat);
-        }
-
         if (structure) {
             d->structures.insert(packageFormat, structure);
             Package p(structure);
@@ -198,41 +128,28 @@ QList<KPluginMetaData> PackageLoader::listPackages(const QString &packageFormat,
     }
 
     for (auto const &plugindir : std::as_const(paths)) {
-        const QDirIterator::IteratorFlags flags = QDirIterator::Subdirectories;
-        const QStringList nameFilters = {QStringLiteral("metadata.json"), QStringLiteral("metadata.desktop")};
-
-        QDirIterator it(plugindir, nameFilters, QDir::Files, flags);
-        QSet<QString> dirs;
+        QDirIterator it(plugindir, QStringList{QStringLiteral("metadata.json")}, QDir::Files, QDirIterator::Subdirectories);
+        std::unordered_set<QString> dirs;
         while (it.hasNext()) {
             it.next();
 
             const QString dir = it.fileInfo().absoluteDir().path();
-
-            if (dirs.contains(dir)) {
+            if (!dirs.insert(dir).second) {
                 continue;
             }
-            dirs << dir;
 
             const QString metadataPath = it.fileInfo().absoluteFilePath();
-            KPluginMetaData info;
-#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 92)
-            if (metadataPath.endsWith(QLatin1String(".desktop"))) {
-                info = KPluginMetaData::fromDesktopFile(metadataPath);
-            } else {
-                info = KPluginMetaData::fromJsonFile(metadataPath);
-            }
-#else
-            info = KPluginMetaData::fromJsonFile(metadataPath);
-#endif
+            KPluginMetaData info = KPluginMetaData::fromJsonFile(metadataPath);
 
             if (!info.isValid() || uniqueIds.contains(info.pluginId())) {
                 continue;
             }
 
-            const QStringList kpackageTypes = readKPackageTypes(info);
-            if (packageFormat.isEmpty() || kpackageTypes.isEmpty() || kpackageTypes.contains(packageFormat)) {
+            if (packageFormat.isEmpty() || readKPackageType(info) == packageFormat) {
                 uniqueIds << info.pluginId();
                 lst << info;
+            } else {
+                qInfo() << "KPackageStructure of" << info << "does not match requested format" << packageFormat;
             }
         }
     }
@@ -260,10 +177,10 @@ KPackage::PackageStructure *PackageLoader::loadPackageStructure(const QString &p
 {
     PackageStructure *structure = d->structures.value(packageFormat).data();
     if (!structure) {
-        if (packageFormat == QStringLiteral("KPackage/Generic")) {
+        if (packageFormat == QLatin1String("KPackage/Generic")) {
             structure = new GenericPackage();
             d->structures.insert(packageFormat, structure);
-        } else if (packageFormat == QStringLiteral("KPackage/GenericQML")) {
+        } else if (packageFormat == QLatin1String("KPackage/GenericQML")) {
             structure = new GenericQMLPackage();
             d->structures.insert(packageFormat, structure);
         }
@@ -274,17 +191,14 @@ KPackage::PackageStructure *PackageLoader::loadPackageStructure(const QString &p
     }
 
     const KPluginMetaData metaData = structureForKPackageType(packageFormat);
-
-    QString error;
     if (!metaData.isValid()) {
         qCWarning(KPACKAGE_LOG) << "Invalid metadata for package structure" << packageFormat;
         return nullptr;
     }
 
-    auto result = KPluginFactory::instantiatePlugin<PackageStructure>(metaData, nullptr, {metaData.rawData().toVariantMap()});
-
+    auto result = KPluginFactory::instantiatePlugin<PackageStructure>(metaData);
     if (!result) {
-        qCWarning(KPACKAGE_LOG) << i18n("Could not load installer for package of type %1. Error reported was: %2", packageFormat, result.errorString);
+        qCWarning(KPACKAGE_LOG).noquote() << "Could not load installer for package of type" << packageFormat << "Error reported was: " << result.errorString;
         return nullptr;
     }
 
@@ -300,12 +214,9 @@ void PackageLoader::addKnownPackageStructure(const QString &packageFormat, KPack
     d->structures.insert(packageFormat, structure);
 }
 
-#if KPACKAGE_BUILD_DEPRECATED_SINCE(5, 86)
-Package PackageLoader::internalLoadPackage(const QString &name)
+void PackageLoader::invalidateCache()
 {
-    Q_UNUSED(name);
-    return Package();
+    self()->d->maxCacheAge = -1;
 }
-#endif
 
 } // KPackage Namespace
