@@ -1,21 +1,8 @@
 /*
- *   Copyright 2010 by Ryan Rix <ry@n.rix.si>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2010 Ryan Rix <ry@n.rix.si>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef KPACKAGE_LOADER_H
 #define KPACKAGE_LOADER_H
@@ -26,10 +13,11 @@
 
 namespace KPackage
 {
-
 class PackageLoaderPrivate;
 
 /**
+ * @class PackageLoader kpackage/packageloader.h <KPackage/PackageLoader>
+ *
  * This is an abstract base class which defines an interface to which the package
  * loading logic can communicate with a parent application. The plugin loader
  * must be set before any plugins are loaded, otherwise (for safety reasons), the
@@ -39,7 +27,7 @@ class PackageLoaderPrivate;
  *
  * @author Ryan Rix <ry@n.rix.si>
  **/
-class PACKAGE_EXPORT PackageLoader
+class KPACKAGE_EXPORT PackageLoader
 {
 public:
     /**
@@ -56,7 +44,7 @@ public:
 
     /**
      * List all available packages of a certain type
-     * 
+     *
      * @param packageFormat the format of the packages to list
      * @param packageRoot the root folder where the packages are installed.
      *          If not specified the default from the packageformat will be taken.
@@ -77,13 +65,18 @@ public:
      * @return metadata for all the matching packages
      * @since 5.10
      */
-    QList<KPluginMetaData> findPackages(const QString &packageFormat, const QString &packageRoot = QString(), std::function<bool(const KPluginMetaData &)> filter = std::function<bool(const KPluginMetaData &)>());
+    QList<KPluginMetaData> findPackages(const QString &packageFormat,
+                                        const QString &packageRoot = QString(),
+                                        std::function<bool(const KPluginMetaData &)> filter = std::function<bool(const KPluginMetaData &)>());
 
     /**
      * Loads a PackageStructure for a given format. The structure can then be used as
      * paramenter for a Package instance constructor
+     *
+     * @note The returned pointer is managed by KPackage, and should never be deleted
+     *
      * @param packageFormat the package format, such as "KPackage/GenericQML"
-     * @return the structure instance
+     * @return the structure instance (ownership retained by KPackage)
      */
     KPackage::PackageStructure *loadPackageStructure(const QString &packageFormat);
 
@@ -96,39 +89,20 @@ public:
      */
     void addKnownPackageStructure(const QString &packageFormat, KPackage::PackageStructure *structure);
 
-    /**
-     * Set the plugin loader which will be queried for all loads.
-     *
-     * @param loader A subclass of PackageLoader which will be supplied
-     * by the application
-     **/
-    static void setPackageLoader(PackageLoader *loader);
-
     /**
      * Return the active plugin loader
      **/
     static PackageLoader *self();
 
 protected:
-
-    /**
-     * A re-implementable method that allows subclasses to override
-     * the default behaviour of loadPackage. If the service requested is not recognized,
-     * then the implementation should return an empty and invalid Package(). 
-     * This method is called
-     * by loadPackage prior to attempting to load a Package using the standard
-     * plugin mechanisms.
-     *
-     * @param packageFormat the format of the package to load
-     *
-     * @return a Package instance with the proper PackageStructure
-     **/
-    virtual Package internalLoadPackage(const QString &packageFormat);
-
     PackageLoader();
     virtual ~PackageLoader();
 
 private:
+    friend class Package;
+    friend class PackageJob;
+    KPACKAGE_NO_EXPORT static void invalidateCache();
+
     PackageLoaderPrivate *const d;
     Q_DISABLE_COPY(PackageLoader)
 };
