@@ -9,7 +9,8 @@
 #define KPACKAGE_PACKAGEJOBTHREAD_P_H
 
 #include "package.h"
-#include <QThread>
+#include "packagejob.h"
+#include <QRunnable>
 
 namespace KPackage
 {
@@ -17,31 +18,23 @@ class PackageJobThreadPrivate;
 
 bool indexDirectory(const QString &dir, const QString &dest);
 
-// true if version2 is more recent than version1
-// TODO: replace with QVersionNumber when we will be able to depend from Qt 5.6
-bool isVersionNewer(const QString &version1, const QString &version2);
-
-class PackageJobThread : public QThread
+class PackageJobThread : public QObject, public QRunnable
 {
     Q_OBJECT
-
 public:
-    enum OperationType {
-        Install,
-        Update,
-    };
+    explicit PackageJobThread(PackageJob::OperationType type, const QString &src, const QString &dest, const QString &packagePath);
+    ~PackageJobThread() override;
 
-    explicit PackageJobThread(QObject *parent = nullptr);
-    virtual ~PackageJobThread();
+    void run() override;
 
     bool install(const QString &src, const QString &dest);
     bool update(const QString &src, const QString &dest);
     bool uninstall(const QString &packagePath);
 
-    Package::JobError errorCode() const;
+    PackageJob::JobError errorCode() const;
 
 Q_SIGNALS:
-    void jobThreadFinished(bool success, const QString &errorMessage = QString());
+    void jobThreadFinished(bool success, PackageJob::JobError errorCode, const QString &errorMessage = QString());
     void percentChanged(int percent);
     void error(const QString &errorMessage);
     void installPathChanged(const QString &installPath);
@@ -50,7 +43,7 @@ private:
     // OperationType says whether we want to install, update or any
     // new similar operation it will be expanded
     bool installDependency(const QUrl &src);
-    bool installPackage(const QString &src, const QString &dest, OperationType operation);
+    bool installPackage(const QString &src, const QString &dest, PackageJob::OperationType operation);
     bool uninstallPackage(const QString &packagePath);
     PackageJobThreadPrivate *d;
 };
